class Attachment < ActiveRecord::Base

  hobo_model # Don't put anything above this

  fields do
    timestamps
  end
  
  belongs_to :client
  
  # --- Paperclip --- #
  has_attached_file :file, 
      :whiny => false, 
      :path => "#{RAILS_ROOT}/files/:id.:extension"
  validates_attachment_size :file, :less_than => 5.megabytes
  validates_attachment_presence :file

  # --- Extra attributes --- #
  def name
    file.original_filename
  end
  
  def size
    if (file_file_size / 1024) < 1024
      (file_file_size / 1024).to_s + ' KB'
    else
      (file_file_size / 1048576).to_s + ' MB'
    end
  end


  # --- Permissions --- #

  def create_permitted?
    acting_user.administrator?
  end

  def update_permitted?
    acting_user.administrator?
  end

  def destroy_permitted?
    acting_user.administrator?
  end

  def view_permitted?(field)
    true
  end

end
